<?php

/*
 *  restclient-php is a full-featured REST client for PUMA and/or
 *  BibSonomy.
 *
 *  Copyright (C) 2015
 *
 *  Knowledge & Data Engineering Group,
 *  University of Kassel, Germany
 *  http://www.kde.cs.uni-kassel.de/
 *
 *  HothoData GmbH, Germany
 *  http://www.academic-puma.de
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace AcademicPuma\RestClient\Model;

use AcademicPuma\RestClient\Util\BibtexModelUtils;
use AcademicPuma\RestClient\Exceptions;
use AcademicPuma\RestClient\Util\StringUtils;

/**
 * Description of Publication
 *
 * @author Sebastian Böttger <boettger@cs.uni-kassel.de>
 */
class Bibtex extends Resource {

    private $bibtexKey;
    private $bKey;
    private $misc;
    private $bibtexAbstract;
    private $entrytype;
    private $address;
    private $annote;
    private $author;
    private $booktitle;
    private $chapter;
    private $crossref;
    private $edition;
    private $editor;
    private $howpublished;
    private $institution;
    private $organization;
    private $journal;
    private $note;
    private $number;
    private $pages;
    private $publisher;
    private $school;
    private $series;
    private $volume;
    private $day;
    private $month;
    private $year;
    private $type;
    private $url;
    private $privnote;

    private $miscFields = [];
    private $miscFieldsParsed = false;
    
    public function __construct() {
    }

    /**
     * The title of the work
     * @return string
     */
    public function getTitle() {
        return $this->title;
    }

    /**
     * @return bool
     */
    public function hasTitle() {
        return !empty($this->title);
    }

    /**
     * BibTeX Key is used to cite or cross-reference the publication.
     * @return mixed
     */
    public function getBibtexKey() {
        return $this->bibtexKey;
    }

    /**
     * @return bool
     */
    public function hasBibtexKey() {
        return !empty($this->title);
    }

    /**
     * @return string
     */
    public function getBKey() {
        return $this->bKey;
    }

    /**
     * @return bool
     */
    public function hasBKey() {
        return !empty($this->bKey);
    }

    /**
     * Misc fields can be used to add additional fields which are not defined by BibTeX
     * @return string
     */
    public function getMisc() {
        return $this->misc;
    }

    /**
     * Abstract of the publication
     * @return string
     */
    public function getBibtexAbstract() {
        return $this->bibtexAbstract;
    }

    /**
     * @return bool
     */
    public function hasBibtexAbstract() {
        return !empty($this->bibtexAbstract);
    }

    /**
     * A BibTeX can contain a different types of entries
     * @return string
     */
    public function getEntrytype() {
        return $this->entrytype;
    }

    /**
     * @return bool
     */
    public function hasEntrytype() {
        return !empty($this->entrytype);
    }

    /**
     * Publisher's address (usually just the city, but can be the full address for lesser-known publishers)
     * @return string
     */
    public function getAddress() {
        return $this->address;
    }

    /**
     * @return bool
     */
    public function hasAddress() {
        return !empty($this->address);
    }

    /**
     * An annotation for annotated bibliography styles (not typical)
     * @return string
     */
    public function getAnnote() {
        return $this->annote;
    }

    /**
     * @return bool
     */
    public function hasAnnote() {
        return !empty($this->annote);
    }

    /**
     * The name(s) of the author(s) (in the case of more than one author, separated by and
     * @return string
     */
    public function getAuthor() {
        return $this->author;
    }

    /**
     * @return bool
     */
    public function hasAuthor() {
        return !empty($this->author);
    }

    /**
     * The title of the book, if only part of it is being cited
     * @return string
     */
    public function getBooktitle() {
        return $this->booktitle;
    }

    /**
     * @return bool
     */
    public function hasBooktitle() {
        return !empty($this->booktitle);
    }

    /**
     * The chapter number
     * @return string
     */
    public function getChapter() {
        return $this->chapter;
    }

    /**
     * @return bool
     */
    public function hasChapter() {
        return !empty($this->chapter);
    }

    /**
     * The key of the cross-referenced entry
     * @return string
     */
    public function getCrossref() {
        return $this->crossref;
    }

    /**
     * @return bool
     */
    public function hasCrossref() {
        return !empty($this->crossref);
    }

    /**
     * The edition of a book, long form (such as "First" or "Second")
     * @return string
     */
    public function getEdition() {
        return $this->edition;
    }

    /**
     * @return bool
     */
    public function hasEdition() {
        return !empty($this->edition);
    }

    /**
     * The name(s) of the editor(s)
     * @return string
     */
    public function getEditor() {
        return $this->editor;
    }

    /**
     * @return bool
     */
    public function hasEditor() {
        return !empty($this->editor);
    }

    /**
     * How it was published, if the publishing method is nonstandard
     * @return string
     */
    public function getHowpublished() {
        return $this->howpublished;
    }

    /**
     * @return bool
     */
    public function hasHowpublished() {
        return !empty($this->howpublished);
    }

    /**
     * The institution that was involved in the publishing, but not necessarily the publisher
     * @return string
     */
    public function getInstitution() {
        return $this->institution;
    }

    /**
     * @return bool
     */
    public function hasInstitution() {
        return !empty($this->institution);
    }

    /**
     * The conference sponsor
     * @return string
     */
    public function getOrganization() {
        return $this->organization;
    }

    /**
     * @return bool
     */
    public function hasOrganization() {
        return !empty($this->organization);
    }

    /**
     * The journal or magazine the work was published in
     * @return string
     */
    public function getJournal() {
        return $this->journal;
    }

    /**
     * @return bool
     */
    public function hasJournal() {
        return !empty($this->journal);
    }

    /**
     * Miscellaneous extra information
     * @return string
     */
    public function getNote() {
        return $this->note;
    }

    /**
     * @return bool
     */
    public function hasNote() {
        return !empty($this->note);
    }

    /**
     * The "(issue) number" of a journal, magazine, or tech-report, if applicable. (Most publications have a "volume",
     * but no "number" field.)
     * @return string
     */
    public function getNumber() {
        return $this->number;
    }

    /**
     * @return bool
     */
    public function hasNumber() {
        return !empty($this->number);
    }

    /**
     * Page numbers, separated either by commas or double-hyphens.
     * @return string
     */
    public function getPages() {
        return $this->pages;
    }

    /**
     * @return bool
     */
    public function hasPages() {
        return !empty($this->pages);
    }

    /**
     * The publisher's name
     * @return string
     */
    public function getPublisher() {
        return $this->publisher;
    }

    /**
     * @return bool
     */
    public function hasPublisher() {
        return !empty($this->publisher);
    }

    /**
     * The school where the thesis was written
     * @return string
     */
    public function getSchool() {
        return $this->school;
    }

    /**
     * @return bool
     */
    public function hasSchool() {
        return !empty($this->school);
    }

    /**
     * The series of books the book was published in (e.g. "The Hardy Boys" or "Lecture Notes in Computer Science")
     * @return string
     */
    public function getSeries() {
        return $this->series;
    }

    /**
     * @return bool
     */
    public function hasSeries() {
        return !empty($this->series);
    }

    /**
     * The volume of a journal or multi-volume book
     * @return string
     */
    public function getVolume() {
        return $this->volume;
    }

    /**
     * @return bool
     */
    public function hasVolume() {
        return !empty($this->volume);
    }

    /**
     * The month of publication (or, if unpublished, the day of creation)
     * @return string
     */
    public function getDay() {
        return $this->day;
    }

    /**
     * @return bool
     */
    public function hasDay() {
        return !empty($this->day);
    }

    /**
     * The month of publication (or, if unpublished, the month of creation)
     * @return string
     */
    public function getMonth() {
        return $this->month;
    }

    /**
     * @return bool
     */
    public function hasMonth() {
        return !empty($this->month);
    }

    /**
     * The year of publication (or, if unpublished, the year of creation)
     * @return string
     */
    public function getYear() {
        return $this->year;
    }

    /**
     * @return bool
     */
    public function hasYear() {
        return !empty($this->year);
    }

    /**
     * The field overriding the default type of publication (e.g. "Research Note" for techreport, "{PhD} dissertation"
     * for phdthesis, "Section" for inbook/incollection)
     * @return string
     */
    public function getType() {
        return $this->type;
    }

    /**
     * @return bool
     */
    public function hasType() {
        return !empty($this->type);
    }

    /**
     * URL of the publication
     * @return string
     */
    public function getUrl() {
        return $this->url;
    }

    /**
     * @return bool
     */
    public function hasUrl() {
        return !empty($this->url);
    }

    /**
     * Private note of the publication
     * @return string
     */
    public function getPrivnote() {
        return $this->privnote;
    }

    /**
     * @return bool
     */
    public function hasPrivnote() {
        return !empty($this->privnote);
    }

    /**
     * @param string $title
     *
     * @return $this
     */
    public function setTitle($title) {
        $this->title = $title;
        return $this;
    }

    /**
     * @param string $bibtexKey
     *
     * @return $this
     */
    public function setBibtexKey($bibtexKey) {
        $this->bibtexKey = $bibtexKey;
        return $this;
    }

    /**
     * @param string $bKey
     *
     * @return $this
     */
    public function setBKey($bKey) {
        $this->bKey = $bKey;
        return $this;
    }

    /**
     * @param string $misc
     *
     * @return $this
     */
    public function setMisc($misc) {
        $this->miscFieldsParsed = false;
        $this->misc = $misc;
        return $this;
    }

    /**
     * @param string $bibtexAbstract
     *
     * @return $this
     */
    public function setBibtexAbstract($bibtexAbstract) {
        $this->bibtexAbstract = $bibtexAbstract;
        return $this;
    }

    /**
     * @param $entrytype
     *
     * @return $this
     */
    public function setEntrytype($entrytype) {
        $this->entrytype = $entrytype;
        return $this;
    }

    /**
     * @param string $address
     *
     * @return $this
     */
    public function setAddress($address) {
        $this->address = $address;
        return $this;
    }

    /**
     * @param string $annote
     *
     * @return $this
     */
    public function setAnnote($annote) {
        $this->annote = $annote;
        return $this;
    }

    /**
     * @param string $author
     *
     * @return $this
     */
    public function setAuthor($author) {
        $this->author = $author;
        return $this;
    }

    /**
     * @param string $booktitle
     *
     * @return $this
     */
    public function setBooktitle($booktitle) {
        $this->booktitle = $booktitle;
        return $this;
    }

    /**
     * @param string $chapter
     *
     * @return $this
     */
    public function setChapter($chapter) {
        $this->chapter = $chapter;
        return $this;
    }

    /**
     * @param string $crossref
     *
     * @return $this
     */
    public function setCrossref($crossref) {
        $this->crossref = $crossref;
        return $this;
    }

    /**
     * @param string $edition
     *
     * @return $this
     */
    public function setEdition($edition) {
        $this->edition = $edition;
        return $this;
    }

    /**
     * @param string $editor
     *
     * @return $this
     */
    public function setEditor($editor) {
        $this->editor = $editor;
        return $this;
    }

    /**
     * @param string $howpublished
     *
     * @return $this
     */
    public function setHowpublished($howpublished) {
        $this->howpublished = $howpublished;
        return $this;
    }

    /**
     * @param string $institution
     *
     * @return $this
     */
    public function setInstitution($institution) {
        $this->institution = $institution;
        return $this;
    }

    /**
     * @param string $organization
     *
     * @return $this
     */
    public function setOrganization($organization) {
        $this->organization = $organization;
        return $this;
    }

    /**
     * @param string $journal
     *
     * @return $this
     */
    public function setJournal($journal) {
        $this->journal = $journal;
        return $this;
    }

    /**
     * @param string $note
     *
     * @return $this
     */
    public function setNote($note) {
        $this->note = $note;
        return $this;
    }

    /**
     * @param string $number
     *
     * @return $this
     */
    public function setNumber($number) {
        $this->number = $number;
        return $this;
    }

    /**
     * @param string $pages
     *
     * @return $this
     */
    public function setPages($pages) {
        $this->pages = $pages;
        return $this;
    }

    /**
     * @param string $publisher
     *
     * @return $this
     */
    public function setPublisher($publisher) {
        $this->publisher = $publisher;
        return $this;
    }

    /**
     * @param string $school
     *
     * @return $this
     */
    public function setSchool($school) {
        $this->school = $school;
        return $this;
    }

    /**
     * @param string $series
     *
     * @return $this
     */
    public function setSeries($series) {
        $this->series = $series;
        return $this;
    }

    /**
     * @param string $volume
     *
     * @return $this
     */
    public function setVolume($volume) {
        $this->volume = $volume;
        return $this;
    }

    /**
     * @param string $day
     *
     * @return $this
     */
    public function setDay($day) {
        $this->day = $day;
        return $this;
    }

    /**
     * @param string $month
     *
     * @return $this
     */
    public function setMonth($month) {
        $this->month = $month;
        return $this;
    }

    /**
     * @param string $year
     *
     * @return $this
     */
    public function setYear($year) {
        $this->year = $year;
        return $this;
    }

    /**
     * @param string $type
     *
     * @return $this
     */
    public function setType($type) {
        $this->type = $type;
        return $this;
    }

    /**
     * @param string $url
     *
     * @return $this
     */
    public function setUrl($url) {
        $this->url = $url;
        return $this;
    }

    /**
     * @param string $privnote
     *
     * @return $this
     */
    public function setPrivnote($privnote) {
        $this->privnote = $privnote;
        return $this;
    }

    /**
     * @param string $prop
     *
     * @return bool
     */
    public function hasMiscField($prop) {
        if (!$this->miscFieldsParsed) {
            $this->miscFields = BibtexModelUtils::parseMiscFieldString($this->getMisc());
            $this->miscFieldsParsed = true;
        }
        return array_key_exists($prop, $this->miscFields) && !empty($this->miscFields[$prop]);
    }

    /**
     * Returns the value of a specified field you want to get from misc. Returns nulls, if the misc field does
     * not exist.
     *
     * @param string $prop which field you want to get
     *
     * @return null|string
     */
    public function getMiscField($prop) {
        if (!$this->miscFieldsParsed) {
            $this->miscFields = BibtexModelUtils::parseMiscFieldString($this->getMisc());
            $this->miscFieldsParsed = true;
        }

        if(array_key_exists($prop, $this->miscFields)) {
            return $this->miscFields[$prop];
        }

        return null;
    }

    /**
     *
     * @return string
     */
    public function __toString() {
        return 'bibtex/' . parent::__toString();
    }

    /**
     * strcasecmp method returns < 0 if str1 is less than str2; > 0 if str1 is greater than str2, and 0 if they are equal.
     * E.g. strcmp('a', 'B') returns -1
     *
     * @param Resource $b
     * @param string   $compareAttribute
     *
     * @return int
     */
    public function compare(Resource $b, $compareAttribute) {

        switch($compareAttribute){
        case 'author':
            $aAuthor = $this->getAuthor() === '' ? $this->getEditor(): $this->getAuthor();
            $bAuthor = $b->getAuthor() === '' ? $this->getEditor() : $b->getAuthor();

            return strcasecmp(StringUtils::transliterateString($aAuthor), StringUtils::transliterateString($bAuthor));
        case 'title':
            return strcasecmp($this->getTitle(), $b->getTitle());
        case 'year':
            return $this->getYear() > $b->getYear() ? 1 : -1;
        case 'entrytype':
            return strcasecmp($this->getEntrytype(), $b->getEntrytype());
        default:
            return 0;
        }
    }

    public function compareEntrytype(Bibtex $b, $typeOrder) {

        $key_a = array_keys($typeOrder, $this->getEntrytype())[0];
        $key_b = array_keys($typeOrder, $b->getEntrytype())[0];

        if ($key_a === '' || $key_a === null) {
            $key_a = count($typeOrder);
        }
        if ($key_b === '' || $key_a === null) {
            $key_b = count($typeOrder);
        }

        $ret = ($key_a === $key_b) ? 0 : (($key_a) > ($key_b) ? 1 : -1);
        return $ret;
    }

    /**
     * Comparator for the publication in the yearly grouped publication
     * for the DBLP-like sorting
     * @param Bibtex $b
     * @param $typeOrder
     *
     * @return int
     */
    public function compareDBLPFormat(Bibtex $b) {

        // Secondary: Compare entry type
        $entryCmp = strcasecmp($this->getEntrytype(), $b->getEntrytype());
        if ($entryCmp === 0) {
            // Tertiary: Compare journal/conference, if both have same entrytype
            $journalCmp = strcasecmp($this->getJournal(), $b->getJournal());
            if ($journalCmp === 0) {
                // Quaternary: Compare pages, if both are in the same journal/conference
                $pagesCmp = strcasecmp($this->getPages(), $b->getPages());
                return $pagesCmp;
            } else {
                return $journalCmp;
            }
        } else {
            return $entryCmp;
        }
    }

    /**
     * Returns the amount of pages in a pages field of the bibtex,
     * for example '123-223' returns 100.
     *
     * @param $pages
     * @return int
     */
    private function getAmountOfPages($pages) {
        if (strpos($pages, '--') !== false) {
            $start = intval(substr($pages, 0, strpos($pages, '--')));
            $end = intval(substr($pages, strpos($pages, '--') + 2));
            return $end - $start;
        } else if (strpos($pages, '-') !== false) {
            $start = intval(substr($pages, 0, strpos($pages, '-')));
            $end = intval(substr($pages, strpos($pages, '-') + 1));
            return $end - $start;
        } else {
            return 1;
        }
    }
}
