<?php

/*
 *  restclient-php is a full-featured REST client for PUMA and/or
 *  BibSonomy.
 *
 *  Copyright (C) 2015
 *
 *  Knowledge & Data Engineering Group,
 *  University of Kassel, Germany
 *  http://www.kde.cs.uni-kassel.de/
 *
 *  HothoData GmbH, Germany
 *  http://www.academic-puma.de
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace AcademicPuma\RestClient\Queries;

use AcademicPuma\CiteProc\CiteProc;
use AcademicPuma\RestClient\Queries\Get\GetPopularPostsQuery;
use AcademicPuma\RestClient\Queries\Get\GetPostDetailsQuery;
use AcademicPuma\RestClient\Queries\Get\GetPostsQuery;
use AcademicPuma\RestClient\Model;
use AcademicPuma\RestClient\Renderer;
use AcademicPuma\RestClient\Exceptions;
use AcademicPuma\RestClient\Util;
use AcademicPuma\RestClient\Config\RESTConfig;
use GuzzleHttp\Client;

/**
 * An abstract query from which all other queries inherit.
 * Queries prepare URL's for API calls, send them and hold the response.
 *
 * @author Florian Fassing
 * @author Sebastian Böttger
 */
abstract class AbstractQuery {

    /**
     * Utility class for building url's.
     *
     * @var \AcademicPuma\RestClient\util\UrlBuilder
     */
    protected $urlBuilder;

    /**
     * Built url gets saved in this.
     *
     * @var string
     */
    protected $url;

    /**
     *
     * @var \GuzzleHttp\Message\ResponseInterface
     */
    protected $response;

    /**
     * @var Model\ModelObject
     */
    protected $model;

    /**
     * @var string
     */
    protected $csl;

    /**
     * @var string
     */
    protected $bibtex = '';

    /**
     * @var string
     */
    protected $endnote = '';

    /**
     * @var string
     */
    protected $bibliography = '';

    /**
     *
     * @var bool
     */
    protected $executed;

    protected $body;



    public function __construct() {
        $this->executed = false;
        $this->urlBuilder = new Util\UrlBuilder();
        $this->client = new Client();
    }

    /**
     * Executes the query and returns a response
     *
     * @param Client $client
     * @param array $reqOpts Request options containing body, headers or curl options.
     * @return self;
     */
    public abstract function execute(Client $client, array $reqOpts);


    /**
     * @return string
     */
    public function getUrl() {

        return $this->url;
    }

    /* ATTENTION
     * The following functions are not tested yet.
     *
     */

    /**
     * @return integer
     */
    public function getStatusCode() {
        return intval($this->response->getStatusCode());
    }

    /**
     * @return bool
     */
    public function isExecuted() {
        return $this->executed;
    }

    /**
     * @return array
     */
    public function getHeaders() {
        return $this->response->getHeaders();
    }

    /**
     * @return string
     */
    public function getReasonPhrase() {
        return $this->response->getReasonPhrase();
    }

    /**
     * @return string
     * @throws Exceptions\UnsupportedOperationException
     */
    public function getBody() {

        if ($this->executed === false) {
            throw new Exceptions\UnsupportedOperationException("Cannot access response, first execute query.");
        }

        if (empty($this->body)) {
            $this->body = $this->response->getBody()->getContents();
        }

        return $this->body;
    }

    /**
     * @param integer $treatCurlyBraces Determines how to treat curly braces in title, abstract and author field.
     * @param integer $treatBackslashes Determines how to treat backslashes in title, abstract and author field.
     * @param boolean $bibTexCleaning Determines whether BibTex cleaning will be executed.
     * @return Model\ModelObject
     * @throws Exceptions\UnsupportedOperationException
     */
    public function model($treatCurlyBraces = RESTConfig::CB_REMOVE, $treatBackslashes = RESTConfig::BS_KEEP_IN_MATH_MODE, $bibTexCleaning = true) {

        if ($this->executed === false) {
            throw new Exceptions\UnsupportedOperationException("Cannot create Model, first execute query.");
        }

        if (empty($this->model)) {
            $this->model = (new Renderer\XMLModelUnserializer($this->getBody(), $treatCurlyBraces, $treatBackslashes, $bibTexCleaning))
                ->convertToModel();
        }
        return $this->model;
    }

    /**
     * @return array
     * @throws Exceptions\UnsupportedOperationException
     */
    public function csl() {

        if ($this->executed === false) {
            throw new Exceptions\UnsupportedOperationException("Cannot create CSL, first execute query.");
        }

        if (!($this instanceof GetPostsQuery) && !($this instanceof GetPopularPostsQuery) && !($this instanceof GetPostDetailsQuery)) {
            throw new Exceptions\UnsupportedOperationException("Method csl() not support for " . get_class($this) . ".");
        }

        if (empty($this->csl)) {
            $this->csl = (new Renderer\CSLModelRenderer())->render($this->model());
        }

        return $this->csl;
    }

    /**
     * @param string $bibliographyStyleName supported styles: 'apa', 'din-1505-2', 'ieee', 'harvard1', 'harvard7de', 'vancouver'
     * @param string $lang
     * @param string $bibliographyStyle
     *
     * @return string
     * @throws Exceptions\FileNotFoundException
     * @throws Exceptions\UnsupportedOperationException
     */
    public function bibliography($bibliographyStyleName, $lang = 'en-US', $bibliographyStyle = '') {

        if (empty($this->bibliography)) {

            if (empty($bibliographyStyle) || !is_string($bibliographyStyle)) {
                $bibliographyStyle = CiteProc::loadStyleSheet($bibliographyStyleName);
            }

            if ($bibliographyStyle === false) {
                throw new Exceptions\FileNotFoundException("Could not find a style with the name
                    '$bibliographyStyleName'.");
            }

            $citeProc = new CiteProc($bibliographyStyle, $lang);
            $cslItems = $this->csl();
            $this->bibliography .= '<ol>';
            foreach ($cslItems as $cslItem) {
                $this->bibliography .= '<li>'.$citeProc->render($cslItem).'</li>';
            }
            $this->bibliography .= '</ol>';
        }

        return $this->bibliography;

    }

    /**
     * @return string
     * @throws Exceptions\UnsupportedOperationException
     */
    public function bibtex() {

        if ($this->executed === false) {
            throw new Exceptions\UnsupportedOperationException("Cannot create BibTeX, first execute query.");
        }

        if (!($this instanceof GetPostsQuery) && !($this instanceof GetPopularPostsQuery) && !($this instanceof GetPostDetailsQuery)) {
            throw new Exceptions\UnsupportedOperationException("Method bibtex() not support for " . get_class($this) . ".");
        }

        if (empty($this->bibtex)) {
            $this->bibtex = (new Renderer\BibtexModelRenderer())->render($this->model());
        }

        return $this->bibtex;
    }

    /**
     * @return string
     * @throws Exceptions\UnsupportedOperationException
     */
    public function endnote() {

        if ($this->executed === false) {
            throw new Exceptions\UnsupportedOperationException("Cannot create Endnote, first execute query.");
        }

        if (!($this instanceof GetPostsQuery) && !($this instanceof GetPopularPostsQuery) && !($this instanceof GetPostDetailsQuery)) {
            throw new Exceptions\UnsupportedOperationException("Method endnote() not support for " . get_class($this) . ".");
        }

        if (empty($this->endnote)) {
            $this->endnote = (new Renderer\EndnoteModelRenderer())->render($this->model());
        }

        return $this->endnote;
    }
}
