<?php

/*
 *  restclient-php is a full-featured REST client for PUMA and/or
 *  BibSonomy.
 *
 *  Copyright (C) 2015
 *
 *  Knowledge & Data Engineering Group,
 *  University of Kassel, Germany
 *  http://www.kde.cs.uni-kassel.de/
 *
 *  HothoData GmbH, Germany
 *  http://www.academic-puma.de
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace AcademicPuma\RestClient\Queries\Get;

use AcademicPuma\RestClient\Config;
use AcademicPuma\RestClient\Util;
use AcademicPuma\RestClient\Queries;
use GuzzleHttp\Client;

/**
 * Builds URL for GetPosts REST-API calls.
 * Also holds the response after being executed.
 *
 * @author Sebastian Böttger
 * @author Florian Fassing
 */
class GetPostsQuery extends Queries\AbstractQuery {

    /**
     * The query constructor builds the url and saves it into the url property.
     *
     * GET /users/[username]/posts ?tags=[t1+t2+...+tn] ?resourcetype=(bibtexǀbookmark)
     *
     * @param string   $resourceType   the requested resourcetype
     * @param string   $grouping       determines to which group $groupingName belongs
     * @param string   $groupingName   name of user or group who/which created the posts
     * @param array    $tags           a list of tags filtering the posts
     * @param string   $hash           interhash of resource
     * @param string   $search         full string search currently not implemented
     * @param string   $format         (xml|json|csl|bibtex|endnote)
     * @param int      $start          start index
     * @param int      $end            end index
     */
    public function __construct($resourceType, $grouping, $groupingName, array $tags = [], $hash = null, $search = null, $format = 'xml', $start = 0, $end = 20) {
        parent::__construct();

        // Check whether start and end params make sense.
        if (is_numeric($start) && $start < 0) { $start = 0; }
        if (is_numeric($end ) && $end < $start) { $end = $start; }

        Util\ParameterCheck::checkGrouping($grouping);
        Util\ParameterCheck::checkResourcetype($resourceType);

        $this->url = $this->urlBuilder->buildUrl(
            [Config\RESTConfig::POSTS_URL], // path
            [ // params
                Config\RESTConfig::RESOURCE_TYPE_PARAM => $resourceType,
                $grouping                              => $groupingName,
                Config\RESTConfig::TAGS_PARAM          => $tags,
                Config\RESTConfig::RESOURCE_PARAM      => $hash,
                Config\RESTConfig::SEARCH_PARAM        => $search,
                Config\RESTConfig::START_PARAM         => $start,
                Config\RESTConfig::END_PARAM           => $end,
                Config\RESTConfig::FORMAT_PARAM        => $format
            ]
        );
    }

    /**
     * Execute a prepared query.
     *
     * @param Client $httpClient
     * @param array  $reqOpts request options
     *
     * @return $this
     */
    public function execute(Client $httpClient, array $reqOpts = []) {

        $this->response = $httpClient->get($this->url, $reqOpts);
        $this->executed = true;

        return $this;
    }
}
