<?php

/*
 *  restclient-php is a full-featured REST client for PUMA and/or
 *  BibSonomy.
 *
 *  Copyright (C) 2015
 *
 *  Knowledge & Data Engineering Group,
 *  University of Kassel, Germany
 *  http://www.kde.cs.uni-kassel.de/
 *
 *  HothoData GmbH, Germany
 *  http://www.academic-puma.de
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace AcademicPuma\RestClient\Queries\Get;

use AcademicPuma\RestClient\Config;
use AcademicPuma\RestClient\Exceptions;
use AcademicPuma\RestClient\Queries;
use AcademicPuma\RestClient\Util;
use GuzzleHttp\Client;

/**
 * Builds URL for GetTagRelation REST-API calls.
 * Also holds the response after being executed.
 *
 * @author Florian Fassing
 */
class GetTagRelationQuery extends Queries\AbstractQuery {

    /**
     * The query constructor builds the url and saves it into the url property.
     *
     * GET /tags/[tag] ?relation=(relatedǀsimilarǀsubtagsǀsupertags)
     *
     * @param        $grouping
     * @param        $groupingName
     * @param string $relation can be on of the following: see above
     * @param array  $tags
     * @param string $order
     * @param int    $start    start index
     * @param int    $limit
     *
     * @throws Exceptions\UnsupportedOperationException
     */
    public function __construct($grouping, $groupingName, $relation, $tags = [], $order = Config\TagOrder::ALPHANUMERIC, $start = 0, $limit = 20) {
        parent::__construct();

        // Default relation is 'related'.
        if ($relation !== Config\RESTConfig::RELATED_TAG_RELATION &&
            $relation !== Config\RESTConfig::SIMILAR_TAG_RELATION)  {
            throw new Exceptions\UnsupportedOperationException("Relation method $relation is not supported");
        }

        Util\ParameterCheck::checkGrouping($grouping);
        Util\ParameterCheck::checkTagOrder($order);

        $this->url = $this->urlBuilder->buildUrl(
            [// path
                Config\RESTConfig::TAGS_URL,
                $tags
            ],
            [ // params
                $grouping                         => $groupingName,
                Config\RESTConfig::TAG_REL_PARAM  => $relation,
                Config\RESTConfig::ORDER_PARAM    => $order,
                Config\RESTConfig::START_PARAM    => $start,
                Config\RESTConfig::END_PARAM      => $limit
            ]
        );
    }

    /**
     * Execute a prepared query.
     *
     * @param Client $httpClient
     * @param array  $reqOpts request options
     *
     * @return $this
     */
    public function execute(Client $httpClient, array $reqOpts = []) {

        $this->response = $httpClient->get($this->url, $reqOpts);
        $this->executed = true;

        return $this;
    }
}
