<?php

/*
 *  restclient-php is a full-featured REST client for PUMA and/or
 *  BibSonomy.
 *
 *  Copyright (C) 2015
 *
 *  Knowledge & Data Engineering Group,
 *  University of Kassel, Germany
 *  http://www.kde.cs.uni-kassel.de/
 *
 *  HothoData GmbH, Germany
 *  http://www.academic-puma.de
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace AcademicPuma\RestClient;

use AcademicPuma\RestClient\Accessor\Accessor;
use AcademicPuma\RestClient\Config;
use AcademicPuma\RestClient\Exceptions\UnsupportedOperationException;
use AcademicPuma\RestClient\Logic;
use AcademicPuma\RestClient\Model;
use AcademicPuma\RestClient\Queries;

/**
 * PHP REST-client for BibSonomy API and PUMA API.
 *
 * @author Florian Fassing
 */
class RESTClient implements Logic\LogicInterface, Renderer {

    private $httpClient;
    private $reqOpts;

    /**
     * @var \AcademicPuma\RestClient\Queries\AbstractQuery
     */
    private $query;

    /**
     *
     * @param Accessor  $accessor Holds authentication data.
     * @param array     $reqOpts curl request options
     */
    public function __construct($accessor, array $reqOpts = []) {

        $this->httpClient = $accessor->getClient();
        $this->reqOpts = $reqOpts;
    }

    /**
     *
     * @param string         $filePath
     * @param string         $resourceHash
     * @param string         $userName
     * @param Model\Document $document
     *
     * @return RESTClient
     */
    public function changeDocumentName($filePath, $resourceHash, $userName, Model\Document $document) {

        $this->query = new Queries\Put\ChangeDocumentNameQuery($filePath, $resourceHash, $userName, $document);
        $this->query->execute($this->httpClient, $this->reqOpts);

        return $this;
    }

    /**
     *
     * POST /users/[userName]/concepts/[conceptName]
     *
     * Create a new relation/concept
     * note: if a concept already exists with the given name
     * it will be replaced
     *
     * @param Model\Tag $concept     the tag containing subTags
     * @param string    $conceptName the name of the super tag of the concept
     * @param string    $userName    the user name of the user the concept belongs to.
     *
     * @return RESTClient
     */
    public function createConcept(Model\Tag $concept, $conceptName, $userName) {

        $this->query = new Queries\Post\CreateConceptQuery($concept, $conceptName, $userName);
        $this->query->execute($this->httpClient, $this->reqOpts);

        return $this;
    }

    /**
     *
     * Adds a document to a post.
     *
     * @param string $filePath
     * @param string $resourceHash
     * @param string $userName
     *
     * @return RESTClient
     */
    public function createDocument($filePath, $resourceHash, $userName) {

        $this->query = new Queries\Post\CreatePostDocumentQuery($filePath, $resourceHash, $userName);
        $this->query->execute($this->httpClient, $this->reqOpts);

        return $this;
    }

    /**
     * POST /api/users/[username]/posts
     *
     * Add post(s) to an user's collection.
     *
     * @param Model\Post|Model\Posts $posts    the post(s) to add
     * @param string                 $userName The username under which the posts will be added.
     *
     * @return RESTClient
     */
    public function createPosts($posts, $userName) {

        $this->query = new Queries\Post\CreatePostQuery($posts, $userName);
        $this->query->execute($this->httpClient, $this->reqOpts);

        return $this;
    }

    /**
     *
     * We create a UserRelation of the form (sourceUser, targetUser)\in relation
     * sourceUser should be logged in for this
     *
     * @param Model\User $sourceUser leftHandSide of the relation
     * @param Model\User $targetUser rightHandSide of the relation
     *
     * @return RESTClient
     */
    public function createUserRelationship(Model\User $sourceUser, Model\User $targetUser) {

        $this->query = new Queries\Post\CreateUserRelationshipQuery($sourceUser, $targetUser);
        $this->query->execute($this->httpClient, $this->reqOpts);

        return $this;
    }

    /**
     *
     * DELETE /users/[username]/concepts/[conceptname]
     *
     * Delete an existing concept
     *
     * @param string $conceptName - name of the concept to delete
     * @param string $userName
     *
     * @return RESTClient
     */
    public function deleteConcept($conceptName, $userName) {

        $this->query = new Queries\Delete\DeleteConceptQuery($conceptName, $userName);
        $this->query->execute($this->httpClient, $this->reqOpts);

        return $this;
    }

    /**
     *
     * Deletes an existing document. If the resourceHash is given, the document
     * is assumed to be connected to the corresponding resource (identified by
     * the user name in the document). Otherwise the document is independent of
     * any post.
     *
     * @param string $userName     user name of the post/document owner
     * @param string $resourceHash the intraHash of the post the document belongs to.
     * @param string $fileName     fileName of the document which should be deleted.
     *
     * @return RESTClient
     */
    public function deleteDocument($userName, $resourceHash, $fileName) {

        $this->query = new Queries\Delete\DeletePostDocumentQuery($userName, $resourceHash, $fileName);
        $this->query->execute($this->httpClient, $this->reqOpts);

        return $this;
    }

    /**
     * Removes the given post - identified by the connected resource's hash -
     * from the user.
     *
     * @param string $userName     user who's posts are to be removed
     * @param string $resourceHash hash of the resource, which is connected to the post to delete
     *
     * @return RESTClient
     */
    public function deletePosts($userName, $resourceHash) {

        $this->query = new Queries\Delete\DeletePostQuery($userName, $resourceHash);
        $this->query->execute($this->httpClient, $this->reqOpts);

        return $this;
    }

    /**
     *
     * GET /users/[userName]/concepts/[conceptName]
     * GET /concepts/[conceptName]
     *
     * Retrieve Details for a concept, containing the belonging subTags.
     *
     * @param string $conceptName the supertag of the concept
     * @param string $userName    the user name of the user the concept belongs to. If null, system-wide relations will
     *                            be returned.
     *
     * @return RESTClient
     */
    public function getConceptDetails($conceptName, $userName) {

        $this->query = new Queries\Get\GetConceptDetailsQuery($conceptName, $userName);
        $this->query->execute($this->httpClient, $this->reqOpts);

        return $this;
    }

    /**
     * /concepts
     *
     * Retrieve relations
     *
     * @param string $resourceType the requested resourcetype
     * @param string $grouping     grouping entity
     * @param string $groupingName the grouping name
     * @param string $regex        a regex to possibly filter the relatons retrieved
     * @param array  $tags         a list of tags which shall be part of the relations
     * @param string $status       the conceptstatus, i.e. all, picked or unpicked
     * @param int    $start        start index
     * @param int    $end          end index
     *
     * @return       RESTClient
     */
    public function getConcepts($resourceType, $grouping, $groupingName, $regex, $tags, $status, $start, $end) {

        $this->query = new Queries\Get\GetConceptsQuery($resourceType, $grouping, $groupingName, $regex, $tags, $status,
                                                        $start, $end);
        $this->query->execute($this->httpClient, $this->reqOpts);

        return $this;
    }

    /**
     * Get a document from a publication resource.
     *
     * @param string $userName     name of the publication owner
     * @param string $resourceHash intraHash of the post resource
     * @param string $fileName     file name of the document
     * @param string $type         determines whether a picture or the actual file will be received
     *
     * @return RESTClient
     */
    public function getDocumentFile($userName, $resourceHash, $fileName, $type = Config\DocumentType::FILE) {

        $this->query = new Queries\Get\GetDocumentQuery($userName, $resourceHash, $fileName, $type);
        $this->query->execute($this->httpClient, $this->reqOpts);

        return $this;
    }

    /**
     *
     * /groups/[groupName]
     *
     * Returns details of one group.
     *
     * @param string $groupName
     *
     * @return RESTClient
     */
    public function getGroupDetails($groupName) {

        $this->query = new Queries\Get\GetGroupDetailsQuery($groupName);
        $this->query->execute($this->httpClient, $this->reqOpts);

        return $this;
    }

    /**
     *
     * /groups
     *
     * Returns all groups of the system.
     *
     * @param integer $end
     * @param integer $start
     *
     * @return RESTClient
     */
    public function getGroups($start, $end) {

        $this->query = new Queries\Get\GetGroupListQuery($start, $end);
        $this->query->execute($this->httpClient, $this->reqOpts);

        return $this;
    }

    /**
     * Returns details to a post. A post is uniquely identified by a hash of the
     * corresponding resource and a username.
     *
     * @param string $userName     name of the post-owner
     *
     * @param string $resourceHash hash value of the corresponding resource
     * @param string $format       Format of received post (xml|json|csl|bibtex|endnote).
     *                             Default value is 'xml'. If you want to use the model or any ModelRenderer, please
     *                             keep it empty or use 'xml'.
     *
     * @return RESTClient
     */
    public function getPostDetails($userName, $resourceHash, $format = 'xml') {

        $this->query = new Queries\Get\GetPostDetailsQuery($userName, $resourceHash, $format);
        $this->query->execute($this->httpClient, $this->reqOpts);

        return $this;
    }

    /**
     * @param string $resourceType      resource type (bookmark|bibtex)
     * @param string $grouping          grouping tells whom posts are to be shown: the posts of a
     *                                  user, of a group or of the viewables.
     * @param string $groupingName      name of the grouping. if grouping is user, then its the
     *                                  username. if grouping is set to {@link GroupingEntity#ALL},
     *                                  then its an empty string!
     * @param array  $tags              a set of tags
     * @param string $resourceHash      intraHash value of a resource, if one would like to get a list of
     *                                  all posts belonging to a given resource.
     * @param string $search            free text search
     * @param string $format            Format of received post (xml|json|csl|bibtex|endnote).
     *                                  Default value is 'xml'. If you want to use the model or any ModelRenderer,
     *                                  please keep it empty or use 'xml'.
     * @param int    $start             inclusive start index of the view window
     * @param int    $end               exclusive end index of the view window
     *
     * @return RESTClient
     */
    public function getPosts($resourceType = Config\Resourcetype::BIBTEX, $grouping = Config\Grouping::USER, $groupingName = "", array $tags = [], $resourceHash = "", $search = "", $format = 'xml', $start = 0, $end = 20) {

        $this->query = new Queries\Get\GetPostsQuery($resourceType, $grouping, $groupingName, $tags, $resourceHash,
                                                     $search, $format, $start, $end);
        $this->query->execute($this->httpClient, $this->reqOpts);

        return $this;
    }

    /**
     *
     * /tags/[tag]
     *
     * Returns details about a tag. Those details are:
     * <ul>
     * <li>details about the tag itself, like number of occurrences etc</li>
     * <li>list of subtags</li>
     * <li>list of supertags</li>
     * <li>list of correlated tags</li>
     * </ul>
     *
     * @param string $tagName name of the tag
     *
     * @return RESTClient
     */
    public function getTagDetails($tagName) {

        $this->query = new Queries\Get\GetTagDetailsQuery($tagName);
        $this->query->execute($this->httpClient, $this->reqOpts);

        return $this;
    }

    /**
     *
     * @param int     $grouping
     * @param int     $groupingName
     * @param string  $relation
     * @param array   $tags
     * @param string  $order
     * @param integer $start        start position
     * @param int     $end          end position
     *
     * @return RESTClient
     *
     */
    public function getTagRelation($grouping, $groupingName, $relation, $tags = [], $order = Config\TagOrder::ALPHANUMERIC, $start = 0, $end = 20) {

        $this->query = new Queries\Get\GetTagRelationQuery($grouping, $groupingName, $relation, $tags, $order, $start, $end);
        $this->query->execute($this->httpClient, $this->reqOpts);

        return $this;
    }

    /**
     *
     * /tags ?filter=[regex] ?(user|group|viewable)=[username/groupname] ?order=(frequency|alph)
     *
     * Returns a list of tags which can be filtered.
     *
     * @param string  $grouping
     *                       grouping tells whom tags are to be shown: the tags of a user,
     *                       of a group or of the viewables.
     * @param string  $groupingName
     *                       name of the grouping. if grouping is user, then its the
     *                       username. if grouping is set to {@link GroupingEntity#ALL},
     *                       then its an empty string!
     * @param string  $regex a regular expression used to filter the tagnames
     * @param string  $order
     * @param integer $start
     * @param integer $end
     *
     * @return RESTClient
     */
    public function getTags($grouping, $groupingName, $regex, $order, $start, $end) {

        $this->query = new Queries\Get\GetTagsQuery($grouping, $groupingName, $regex, $order, $start, $end);
        $this->query->execute($this->httpClient, $this->reqOpts);

        return $this;
    }

    /**
     *
     * /users/[username]
     *
     * Returns details about a specified user
     *
     * @param string $userName name of the user we want to get details from
     *
     * @return RESTClient
     */
    public function getUserDetails($userName) {

        $this->query = new Queries\Get\GetUserDetailsQuery($userName);
        $this->query->execute($this->httpClient, $this->reqOpts);

        return $this;
    }

    /**
     *
     * GET    /groups/[groupName]/users
     *
     * @param string  $groupName
     * @param integer $start
     * @param integer $end
     *
     * @return RESTClient
     */
    public function getUserListOfGroup($groupName, $start, $end) {

        $this->query = new Queries\Get\GetUserListOfGroupQuery($groupName, $start, $end);
        $this->query->execute($this->httpClient, $this->reqOpts);

        return $this;
    }

    /**
     * URL: /users
     *
     *
     * Generic method to retrieve lists of users
     *
     * @param integer $start start position
     * @param integer $end   end position
     *
     * @return RESTClient
     */
    public function getUsers($start, $end) {

        $this->query = new Queries\Get\GetUserListQuery($start, $end);
        $this->query->execute($this->httpClient, $this->reqOpts);

        return $this;
    }

    /**
     * PUT /users/[username]/concepts/[conceptname]
     *
     * Update an existing relation/concept
     *
     * @param Model\Tag $concept  the concept to update
     * @param string    $userName name of the user who created the concept
     * @param string    $operation
     *
     * @return RESTClient
     */
    public function updateConcept(Model\Tag $concept, $userName, $operation = null) {

        $this->query = new Queries\Put\ChangeConceptQuery($concept, $userName, $operation);
        $this->query->execute($this->httpClient, $this->reqOpts);

        return $this;
    }

    /**
     * Updates the post(s) in the database.
     *
     * @param Model\ModelObject $posts        post(s) to update
     * @param string            $userName     user name of the post owner
     * @param string            $resourceHash of the updated post
     *
     * @return RESTClient
     */
    public function updatePosts(Model\ModelObject $posts, $userName, $resourceHash) {

        $this->query = new Queries\Put\ChangePostQuery($posts, $userName);
        $this->query->execute($this->httpClient, $this->reqOpts);

        return $this;
    }

    /**
     *
     * Publication List of the requested API Result, rendered in given CSL Style
     *
     * @param string $bibliographyStyleName
     * @param string $lang
     * @param string $bibliographyStyle XML formatted CSL Style
     *
     * @return string
     */
    public function bibliography($bibliographyStyleName = 'apa', $lang = 'en', $bibliographyStyle = null) {

        return $this->query->bibliography($bibliographyStyleName, $lang, $bibliographyStyle);
    }

    /**
     *
     * BibTeX Representation of the requested API Result.
     *
     * @return string
     * @throws Exceptions\UnsupportedOperationException
     */
    public function bibtex() {

        return $this->query->bibtex();
    }

    /**
     *
     * CSL Representation of the requested API Result.
     *
     * @return \stdClass
     * @throws Exceptions\UnsupportedOperationException
     */
    public function csl() {

        return $this->query->csl();
    }

    /**
     *
     * Endnote representation of the requested API Result.
     *
     * @return string
     * @throws Exceptions\UnsupportedOperationException
     */
    public function endnote() {

        return $this->query->endnote();
    }

    /**
     * Model Object representation of the requested API Result.
     *
     * @param integer $treatCurlyBraces Determines how to treat curly braces in title, abstract and author field.
     * @param integer $treatBackslashes Determines how to treat backslashes in title, abstract and author field.
     * @param boolean $bibTexCleaning Determines whether BibTex cleaning will be executed.
     * @return Model\ModelObject
     */
    public function model($treatCurlyBraces = Config\RESTConfig::CB_REMOVE, $treatBackslashes = Config\RESTConfig::BS_KEEP_IN_MATH_MODE, $bibTexCleaning = true) {

        return $this->query->model($treatCurlyBraces, $treatBackslashes, $bibTexCleaning);
    }

    /**
     *
     * XML Representation of the requested API Result.
     *
     * @return string xml formatted string
     * @throws Exceptions\UnsupportedOperationException
     */
    public function xml() {

        return $this->query->getBody();
    }

    /**
     * @return \GuzzleHttp\Stream\StreamInterface
     *
     * @throws UnsupportedOperationException
     */
    public function file() {

        if (!($this->query instanceof Queries\Get\GetDocumentQuery)) {
            throw new UnsupportedOperationException('');
        }

        return $this->query->getStream();
    }

    /**
     *
     * @return string
     * @throws UnsupportedOperationException
     */
    public function getBody() {

        return $this->query->getBody();
    }

    /**
     * Returns the Query object of the last used request. Null, if no request has been called before.
     *
     * @return Queries\AbstractQuery
     */
    public function getQuery() {

        return $this->query;
    }
}
