<?php

/*
 *  restclient-php is a full-featured REST client for PUMA and/or
 *  BibSonomy.
 *
 *  Copyright (C) 2015
 *
 *  Knowledge & Data Engineering Group,
 *  University of Kassel, Germany
 *  http://www.kde.cs.uni-kassel.de/
 *
 *  HothoData GmbH, Germany
 *  http://www.academic-puma.de
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace AcademicPuma\RestClient\Renderer;

use AcademicPuma\RestClient\Model;
use AcademicPuma\RestClient\Exceptions;
use AcademicPuma\RestClient\Util;
/**
 * Creates bibtex string from a model.
 *
 * @since 09.07.15
 * @author Sebastian Böttger / boettger@cs.uni-kassel.de
 */
class BibtexModelRenderer extends ModelRenderer {

    const EQUALS_SIGN = '=';
    const TAB = ' ';
    const NEWLINE = PHP_EOL;
    const COMMA_AND_NEWLINE = ",\n";
    const LAST_COMMA = "/^(.+?),\n$/";

    /**
     * @var bool
     */
    protected $includeMiscField;

    /**
     * @param bool $includeMiscField
     */
    public function __construct($includeMiscField = false) {
        $this->includeMiscField = $includeMiscField;
    }

    /**
     * Serializes <pre>$posts </pre> recursively into its XML representation
     *
     * @param \AcademicPuma\RestClient\Model\Posts $posts
     *
     * @return string BibTeX formatted posts
     */
    protected function serializePosts(Model\Posts $posts) {
        $bibtexPosts = [];
        foreach ($posts as $post) {
            $bibtexPosts[] = $this->serializePost($post);
            //$this->bibtexPost = '';
        }
        return implode(PHP_EOL, $bibtexPosts);
    }

    /**
     * Serializes a <pre>$post</pre> recursively into its XML representation
     *
     * @param \AcademicPuma\RestClient\Model\Post $post
     *
     * @return string
     */
    protected function serializePost(Model\Post $post) {

        return $this->serializeResource($post->getResource());
    }

    /**
     * Serializes a <pre>$resource</pre> into its BibTeX representation
     *
     * @param Model\Resource $resource
     *
     * @return string
     * @throws Exceptions\InvalidModelObjectException
     */
    protected function serializeResource(Model\Resource $resource) {
        if ($resource instanceof Model\Bookmark) {
            throw new Exceptions\InvalidModelObjectException("There is no BibTeX representation for resource type
            Bookmark");
        }
        return $this->serializeBibTex($resource);
    }

    /**
     * @param Model\Bibtex $bibtex
     *
     * @return string
     */
    protected function serializeBibTex(Model\Bibtex $bibtex) {

        $bibtexBody = '';

        $bibtexFields = ["abstract", "address", "annote", "author", "booktitle", "chapter", "crossref", "edition",
                         "editor", "howpublished", "institution", "journal", "misc", "month", "note", "number",
                         "organization", "pages", "publisher", "school", "series", "title", "type", "volume", "year"];

        $reflectedBibtex = new \ReflectionClass($bibtex);

        for ($i = 0; $i < count($bibtexFields); ++$i) {

            $field = $bibtexFields[$i];

            if ($field === 'abstract') {

                if ($bibtex->getBibtexAbstract() != null && $bibtex->getBibtexAbstract() != '') {
                    $bibtexBody .= $this->renderBibTeXAttribute($field, $bibtex->getBibtexAbstract());
                }
                continue;
            }

            if ($field === 'misc') {
                $misc = $bibtex->getMisc();
                if ($this->includeMiscField && !empty($misc)) {
                    $bibtexBody .= $this->renderMiscFields($misc);
                }
                continue;
            }
            $method = $reflectedBibtex->getMethod('get'.ucfirst($field));
            $value = $method->invoke($bibtex);

            if (!empty($value)) {
                $bibtexBody .= $this->renderBibTeXAttribute($field, $value, $bibtexBody);
            }
        }

        $bibtexBody = $this->removeLastComma($bibtexBody);

        $bibtexBody .= self::NEWLINE;
        $bibtexItem = '@'.
                      $bibtex->getEntrytype().
                      $this->addBibTexBrackets(
                          $bibtex->getBibtexKey().
                          self::COMMA_AND_NEWLINE.
                          $bibtexBody
                      ).self::NEWLINE;


        return $bibtexItem;
    }


    /**
     * Serializes a set of <pre>$tags</pre> into its XML representation
     *
     * @param \AcademicPuma\RestClient\Model\Tags $tags
     */
    protected function serializeTags(Model\Tags $tags) {
        // TODO: Implement serializeTags() method.
    }

    /**
     * Serializes a <pre>$tag</pre> into its XML representation
     *
     * @param \AcademicPuma\RestClient\Model\Tag $tag
     */
    protected function serializeTag(Model\Tag $tag) {
        // TODO: Implement serializeTag() method.
    }

    /**
     * Serializes a set of <pre>$users</pre> into its XML representation
     *
     * @param \AcademicPuma\RestClient\Model\Users $users
     *
     * @throws Exceptions\UnsupportedOperationException
     */
    protected function serializeUsers(Model\Users $users) {
        throw new Exceptions\UnsupportedOperationException(__METHOD__." not supported in this Model Renderer.");
    }

    /**
     * Serializes a <pre>$user</pre> into its XML representation
     *
     * @param \AcademicPuma\RestClient\Model\User $user
     *
     * @throws Exceptions\UnsupportedOperationException
     */
    protected function serializeUser(Model\User $user) {
        throw new Exceptions\UnsupportedOperationException(__METHOD__." not supported in this Model Renderer.");
    }

    /**
     * Serializes a set of <pre>$groups</pre> into its XML representation
     *
     * @param \AcademicPuma\RestClient\Model\Groups $groups
     *
     * @throws Exceptions\UnsupportedOperationException
     */
    protected function serializeGroups(Model\Groups $groups) {
        throw new Exceptions\UnsupportedOperationException(__METHOD__." not supported in this Model Renderer.");
    }

    /**
     * Serializes a <pre>$group</pre> into its XML representation
     *
     * @param \AcademicPuma\RestClient\Model\Group $group
     *
     * @throws Exceptions\UnsupportedOperationException
     */
    protected function serializeGroup(Model\Group $group) {
        throw new Exceptions\UnsupportedOperationException(__METHOD__." not supported in this Model Renderer.");
    }

    /**
     * Serializes a set of <pre>$documents</pre> into its XML representation
     *
     * @param \AcademicPuma\RestClient\Model\Documents $documents
     *
     * @throws Exceptions\UnsupportedOperationException
     */
    protected function serializeDocuments(Model\Documents $documents) {
        throw new Exceptions\UnsupportedOperationException(__METHOD__." not supported in this Model Renderer.");
    }

    /**
     * Serializes a <pre>$document</pre> into its XML representation
     *
     * @param \AcademicPuma\RestClient\Model\Document $document
     *
     * @throws Exceptions\UnsupportedOperationException
     */
    protected function serializeDocument(Model\Document $document) {
        throw new Exceptions\UnsupportedOperationException(__METHOD__." not supported in this Model Renderer.");
    }

    /**
     * @param $field
     * @param $value
     *
     * @return string
     */
    protected function renderBibTeXAttribute($field, $value) {

        $value = trim($value); //trim
        $part = self::TAB . ' ' . $field . ' ' . self::EQUALS_SIGN . ' ';

        if (preg_match('/^[1-9][0-9]*$/', $value)) {
            $part .= $value;
        } else {
            $part .= $this->addBibTexBrackets($value);
        }

        $part .= self::COMMA_AND_NEWLINE;

        return $part;
    }

    protected function addBibTexBrackets($string) {
        return '{'.$string.'}';
    }

    /**
     * @param string $misc
     *
     * @return string
     */
    private function renderMiscFields($misc) {
        $miscAttrArray = Util\BibtexModelUtils::parseMiscFieldString($misc);
        $miscAttributes = '';
        foreach ($miscAttrArray as $attrName => $attr) {
            $miscAttributes .= $this->renderBibTeXAttribute($attrName, $attr);
        }
        return $miscAttributes;
    }

    private function removeLastComma($bibtexBody) {

        $cutPos = strpos($bibtexBody, ",\n", strlen($bibtexBody)-2);
        return substr($bibtexBody, 0, $cutPos);
    }

}
