<?php

/*
 *  restclient-php is a full-featured REST client for PUMA and/or
 *  BibSonomy.
 *
 *  Copyright (C) 2015
 *
 *  Knowledge & Data Engineering Group,
 *  University of Kassel, Germany
 *  http://www.kde.cs.uni-kassel.de/
 *
 *  HothoData GmbH, Germany
 *  http://www.academic-puma.de
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace AcademicPuma\RestClient\Renderer;

use AcademicPuma\RestClient\Config\Entrytype;
use AcademicPuma\RestClient\Exceptions\InvalidModelObjectException;
use AcademicPuma\RestClient\Model;
use AcademicPuma\RestClient\Util\BibtexModelUtils;
use AcademicPuma\RestClient\Util\CSLTypeEntrytypeMap;
use AcademicPuma\RestClient\Util\CSLUtils;
use AcademicPuma\RestClient\Exceptions;

/**
 * Converts model object structure into CSL JSON string.
 * @package AcademicPuma\RestClient\Renderer
 */
class CSLModelRenderer extends ModelRenderer {


    private $cslDataObject;

    public function __construct() {
        $this->cslDataObject = new \stdClass();
    }


    protected function serializeResource(Model\Resource $resource) {
        if ($resource instanceof Model\Bookmark) {
            throw new InvalidModelObjectException("There is no CSL representation for resource type Bookmark");
        }
        return $this->serializeBibtex($resource);
    }

    protected function serializePosts(Model\Posts $posts) {
        $cslPosts = [];

        foreach($posts as $post) {
            $cslPosts[] = $this->serializePost($post);
            $this->cslDataObject = new \stdClass();
        }

        return $cslPosts;
    }

    /**
     * Serializes a <pre>$post</pre> recursively into its XML representation
     *
     * @param \AcademicPuma\RestClient\Model\Post $post
     *
     * @return \DOMNode
     */
    protected function serializePost(Model\Post $post) {
        $id = $post->getResource()->getIntraHash() . $post->getUser()->getName();
        $this->cslDataObject->id = $id;
        $documents = $post->getDocuments();
        if (!empty($documents)) {
            $this->cslDataObject->documents = $this->serializeDocuments($documents);
        }

        if (count($post->getTag()) > 0) {
            $this->cslDataObject->keyword = implode(" ", $post->getTag());
        }

        return $this->serializeResource($post->getResource());

    }

    /**
     * @param \AcademicPuma\RestClient\Model\Tags $tags
     *
     * @throws \AcademicPuma\RestClient\Exceptions\UnsupportedOperationException
     */
    protected function serializeTags(Model\Tags $tags) {
        throw new Exceptions\UnsupportedOperationException(__METHOD__." not supported in this Model Renderer.");
    }

    /**
     * Serializes a <pre>$tag</pre> into its XML representation
     *
     * @param \AcademicPuma\RestClient\Model\Tag $tag
     *
     * @return \DOMNode
     */
    protected function serializeTag(Model\Tag $tag) {

    }

    /**
     * @param \AcademicPuma\RestClient\Model\Users $users
     *
     * @throws \AcademicPuma\RestClient\Exceptions\UnsupportedOperationException
     */
    protected function serializeUsers(Model\Users $users) {
        throw new Exceptions\UnsupportedOperationException(__METHOD__." not supported in this Model Renderer.");
    }

    /**
     * @param \AcademicPuma\RestClient\Model\User $user
     *
     * @throws \AcademicPuma\RestClient\Exceptions\UnsupportedOperationException
     */
    protected function serializeUser(Model\User $user) {
        throw new Exceptions\UnsupportedOperationException(__METHOD__." not supported in this Model Renderer.");
    }


    protected function serializeGroups(Model\Groups $groups) {
        throw new Exceptions\UnsupportedOperationException(__METHOD__." not supported in this Model Renderer.");
    }

    /**
     * @param \AcademicPuma\RestClient\Model\Group $group
     *
     * @throws \AcademicPuma\RestClient\Exceptions\UnsupportedOperationException
     */
    protected function serializeGroup(Model\Group $group) {
        throw new Exceptions\UnsupportedOperationException(__METHOD__." not supported in this Model Renderer.");
    }

    /**
     * Serializes a set of <pre>$documents</pre> into its XML representation
     *
     * @param \AcademicPuma\RestClient\Model\Documents $documents
     * @return array
     */
    protected function serializeDocuments(Model\Documents $documents) {

        $ret = [];
        foreach ($documents as $document) {
            $ret[] = $this->serializeDocument($document);
        }
        return $ret;
    }

    /**
     * @param Model\Document $document
     *
     * @return \stdClass
     * @internal param Model\Post $post
     */
    protected function serializeDocument(Model\Document $document) {
        $stdClass = new \stdClass();
        $stdClass->fileHash = "";
        $stdClass->fileName = $document->getFilename();
        $stdClass->md5hash = $document->getMd5hash();
        $stdClass->temp = false;
        $matches = [];
        preg_match('/http:\/\/[^\/]+\/api\/users\/([^\/]+).+/', $document->getHref(), $matches);
        $stdClass->userName = $matches[1];
        return $stdClass;
    }

    /**
     * Renders a <pre>ModelObject $object</pre> recursively into its CSL representation
     *
     * @param \AcademicPuma\RestClient\Model\ModelObject $model Object of type <pre>ModelObject</pre>
     *
     * @return string
     */
    public function render(Model\ModelObject $model) {

        return parent::render($model);
    }

    /**
     * @param \AcademicPuma\RestClient\Model\Bibtex $bibtex
     * @return \stdClass
     */
    private function serializeBibtex(Model\Bibtex $bibtex) {

        $this->mapAuthorsAndEditors($bibtex);

        $this->mapLocationAndAddress($bibtex);

        $this->mapBibtexKey($bibtex);

        $this->mapTitles($bibtex);

        $this->mapPublisher($bibtex);

        $this->mapPublicationTitleAndChapter($bibtex);

        // mapping number/issue
        $this->mapNumberAndIssue($bibtex);

        $this->mapAccessed($bibtex);

        $this->mapDate($bibtex);

        $this->mapPages($bibtex);

        if ($bibtex->hasVolume()) {
            $this->cslDataObject->volume = (BibtexModelUtils::cleanBibtex($bibtex->getVolume()));
        } else {
			 $this->cslDataObject->volume = ""; 
		}
        if ($bibtex->hasUrl()) {
            $this->cslDataObject->URL = (BibtexModelUtils::cleanBibtex($bibtex->getUrl()));
        } else {
			 $this->cslDataObject->URL = ""; 
		}
        if ($bibtex->hasMiscField('status')) {
            $this->cslDataObject->status = (BibtexModelUtils::cleanBibtex($bibtex->getMiscField("status")));
        }
        if ($bibtex->hasMiscField('isbn')) {
            $this->cslDataObject->ISBN = (BibtexModelUtils::cleanBibtex($bibtex->getMiscField("isbn")));
        } else {
            $this->cslDataObject->ISBN = "";
        }
        if ($bibtex->hasMiscField('issn')) {
            $this->cslDataObject->ISSN = (BibtexModelUtils::cleanBibtex($bibtex->getMiscField("issn")));
        } else {
            $this->cslDataObject->ISSN = '';
        }
        if ($bibtex->hasMiscField('revision')) {
            $this->cslDataObject->version = (BibtexModelUtils::cleanBibtex($bibtex->getMiscField("revision")));
        }
        if ($bibtex->hasAnnote()) {
            $this->cslDataObject->annote = (BibtexModelUtils::cleanBibtex($bibtex->getAnnote()));
        } else {
            $this->cslDataObject->annote = "";
        }
        if ($bibtex->hasEdition()) {
            $this->cslDataObject->edition = (BibtexModelUtils::cleanBibtex($bibtex->getEdition()));
        } else {
			 $this->cslDataObject->edition = ""; 
		}
        if ($bibtex->hasBibtexAbstract()) {
            $this->cslDataObject->abstract = ($bibtex->getBibtexAbstract());
        } else {
			 $this->cslDataObject->abstract = "";
		}
        if ($bibtex->hasMiscField('doi')) {
            $this->cslDataObject->DOI = (BibtexModelUtils::cleanBibtex($bibtex->getMiscField("doi")));
        } else {
            $this->cslDataObject->DOI = "";
        }
        if ($bibtex->hasNote()) {
            $this->cslDataObject->note = (BibtexModelUtils::cleanBibtex($bibtex->getNote()));
        } else {
			 $this->cslDataObject->note = ""; 
		}

        if (Entrytype::INPROCEEDINGS === $bibtex->getEntrytype()) {
            $this->cslDataObject->{"collection-editor"} = [];
            $this->cslDataObject->{"container-author"} = [];
            $this->cslDataObject->{"event-date"} = $this->cslDataObject->issued;
            $this->cslDataObject->{"page-first"} = "";
            $this->cslDataObject->status = "";
            $this->cslDataObject->version = "";
        }

        $this->cslDataObject->type = (new CSLTypeEntrytypeMap())->getEntrytype($bibtex->getEntrytype());

        return $this->cslDataObject;
    }


    /**
     * @param Model\Bibtex $bibtex
     */
    private function mapTitles(Model\Bibtex $bibtex) {

        // mapping journal, booktitle and series
        $cleanedJournal = BibtexModelUtils::cleanBibtex($bibtex->getJournal());
        $cleanedBooktitle = BibtexModelUtils::cleanBibtex($bibtex->getBooktitle());
        $cleanedSeries = BibtexModelUtils::cleanBibtex($bibtex->getSeries());

        if (!empty($cleanedJournal)) {
            $containerTitleToUse = $cleanedJournal;
        } else {
            if (!empty($cleanedBooktitle)) {
                $containerTitleToUse  = $cleanedBooktitle;
            } else {
                $containerTitleToUse  = $cleanedSeries;
            }
        }
        if (!empty($containerTitleToUse)) {
            $this->cslDataObject->{"container-title"} = $containerTitleToUse;
            $this->cslDataObject->{"collection-title"} = $cleanedSeries;
        } else {
            $this->cslDataObject->{"container-title"} = "";
        }
    }

    /**
     * @param Model\Bibtex $bibtex
     */
    private function mapPublisher(Model\Bibtex $bibtex) {

        // mapping publisher, techreport, thesis, organization
        $publisher = $bibtex->getPublisher();
        $school = BibtexModelUtils::cleanBibtex($bibtex->getSchool());
        $institution = BibtexModelUtils::cleanBibtex($bibtex->getInstitution());
        $organisation = BibtexModelUtils::cleanBibtex($bibtex->getOrganization());

        if (!empty($publisher)) {
            $this->cslDataObject->publisher = $publisher;

        } else {
            if (Entrytype::TECHREPORT === $bibtex->getEntrytype() && !empty($institution)) {
                $this->cslDataObject->publisher = $institution;

            } else {
                if (Entrytype::PHDTHESIS === $bibtex->getEntrytype() && !empty($school)) {
                    $this->cslDataObject->publischer = $school;
                    $this->cslDataObject->genre = "PhD dissertation";

                } else {
                    if (Entrytype::MASTERTHESIS === $bibtex->getEntrytype() && !empty($school)) {
                        $this->cslDataObject->publisher = $school;
                        $this->cslDataObject->genre = "Master thesis";

                    } else {

                        if (!empty($organisation)) {
                            $this->cslDataObject->publisher = $organisation;
                        }
                    }
                }
            }
        }
    }

    /**
     * @param Model\Bibtex $bibtex
     */
    private function mapNumberAndIssue(Model\Bibtex $bibtex) {

        $cleanedNumber = BibtexModelUtils::cleanBibtex($bibtex->getNumber());
        $cleanedIssue = BibtexModelUtils::cleanBibtex($bibtex->getMiscField("issue"));

        if (!empty($cleanedNumber)) {
            $this->cslDataObject->number = $cleanedNumber;
        } else {
            $this->cslDataObject->number = "";
        }

        if (!empty($cleanedIssue)) {
            $issueToUse = $cleanedIssue;
        } else {
            if (!empty($cleanedNumber)) {
                $issueToUse = $cleanedNumber;
            }
        }
        if (!empty($issueToUse)) {
            $this->cslDataObject->issue = $issueToUse;
        } else {
            $this->cslDataObject->issue = "";
        }
    }

    /**
     * @param Model\Bibtex $bibtex
     */
    private function mapDate(Model\Bibtex $bibtex) {

        // date mapping
        $urlDate = BibtexModelUtils::cleanBibtex($bibtex->getMiscField("urldate"));
        $cleanedDate = BibtexModelUtils::cleanBibtex($bibtex->getMiscField("date"));
        $date = new \stdClass();
        if (Entrytype::ELECTRONIC === $bibtex->getEntrytype() && !empty($urlDate)) {
            $date->raw = $urlDate;
        } else {
            if (!empty($cleanedDate)) {
                $date->raw = $cleanedDate;
                $date->{"event-date"} = $date;
            } else {
                $year = $bibtex->getYear();
                $date->{"date-parts"} = [[$year]];
                $date->literal = $year;

                $cleanedMonth = intval(BibtexModelUtils::cleanBibtex($bibtex->getMonth()));
                //$month = 0;

                if (!empty($cleanedMonth)) {
                    //$month = $cleanedMonth;
                    $date->{"date-parts"} = [[$year]];
                    $date->literal = "$year";
                }

                $cleanedDay = BibtexModelUtils::cleanBibtex($bibtex->getDay());
                if (!empty($cleanedDay)) {
                    $date->{"date-parts"} = [[$year]];
                    $date->literal = "$year";
                }
            }
        }

        $this->cslDataObject->issued = $date;
    }

    /**
     * @param Model\Bibtex $bibtex
     */
    private function mapPages(Model\Bibtex $bibtex) {

        $cleanedPages = BibtexModelUtils::cleanBibtex($bibtex->getPages());

        if (!empty($cleanedPages)) {
            /*
            $this->cslDataObject->page = new \stdClass();
            $match = [];
            if (preg_match('/(\d+)([\-]{0,2})(\d+?)/', $cleanedPages, $match)) {

                if (!empty($match[1])) {
                    $this->cslDataObject->page->{"page-first"} = $match[1];
                } else {
                    if (!empty($match[1]) && !empty($match[2]) && !empty($match[3])) {
                        $this->cslDataObject->page->{"number-of-pages"} = (intval($match[3]) - intval($match[1]));
                    }
                }
            } else if (preg_match('/^(\d+)$/', $cleanedPages, $match)) {
                if (!empty($match[1])) {
                    $this->cslDataObject->page = intval($match[1]);
                }
            }
            */
            $this->cslDataObject->page = $cleanedPages;

        } else {
            $this->cslDataObject->page = "";
        }
    }

    /**
     * @param Model\Bibtex $bibtex
     */
    private function mapAccessed(Model\Bibtex $bibtex) {

        $accessed = BibtexModelUtils::cleanBibtex($bibtex->getMiscField("accessed"));
        if (!empty($accessed)) {
            $accessedDate = new \stdClass();
            $accessedDate->literal = $accessed;
            $this->cslDataObject->accessed = $accessedDate;
        }
    }

    /**
     * @param Model\Bibtex $bibtex
     */
    private function mapPublicationTitleAndChapter(Model\Bibtex $bibtex) {

        // mapping chapter
        $chapter = $bibtex->getChapter();
        if (!empty($chapter)) {
            $this->cslDataObject->{"chapter-number"} = BibtexModelUtils::cleanBibtex($chapter);
        }

        // mapping title
        $title = BibtexModelUtils::cleanBibtex($bibtex->getTitle());
        if (!empty($title)) {
            $this->cslDataObject->title = $title;
        } else {
            // XXX: title is a required field
            $this->cslDataObject->title = $chapter;
        }
    }

    /**
     * @param Model\Bibtex $bibtex
     */
    private function mapBibtexKey(Model\Bibtex $bibtex) {

        // mapping bibtexkey
        $bibtexKey = BibtexModelUtils::cleanBibtex($bibtex->getBibtexKey());
        if (!empty($bibtexKey)) {
            $this->cslDataObject->{"citation-label"} = BibtexModelUtils::cleanBibtex($bibtex->getBibtexKey());
        } else {
            $this->cslDataObject->{"citation-label"} = "";
        }
    }

    /**
     * @param Model\Bibtex $bibtex
     */
    private function mapLocationAndAddress(Model\Bibtex $bibtex) {

        // mapping address

        $cleanedLocation = BibtexModelUtils::cleanBibtex($bibtex->getMiscField("location"));
        $cleanedAddress = BibtexModelUtils::cleanBibtex($bibtex->getAddress());
        if (!empty($cleanedLocation) && $bibtex->hasMiscField("location")) {
            $this->cslDataObject->{"event-place"} = $cleanedLocation;
            $this->cslDataObject->{"publisher-place"} = $cleanedLocation;
        } else {
            if (!empty($cleanedAddress)) {
                $this->cslDataObject->{"event-place"} = $cleanedAddress;
                $this->cslDataObject->{"publisher-place"} = $cleanedAddress;
            } else {
                $this->cslDataObject->{"event-place"} = "";
                $this->cslDataObject->{"publisher-place"} = "";
            }
        }
    }

    /**
     * @param Model\Bibtex $bibtex
     */
    private function mapAuthorsAndEditors(Model\Bibtex $bibtex) {

        $authors = [];
        $editors = [];

        $ed = $bibtex->getEditor();
        $at = $bibtex->getAuthor();

        if (!empty($ed)) {
            $editors = CSLUtils::getCSLNames($bibtex->getEditor());
        }

        if (!empty($at)) {
            $authors = CSLUtils::getCSLNames($bibtex->getAuthor());
        }

        $this->cslDataObject->author = $authors;

        $this->cslDataObject->editor = $editors;
    }
}