<?php

/*
 *  restclient-php is a full-featured REST client for PUMA and/or
 *  BibSonomy.
 *
 *  Copyright (C) 2015
 *
 *  Knowledge & Data Engineering Group,
 *  University of Kassel, Germany
 *  http://www.kde.cs.uni-kassel.de/
 *
 *  HothoData GmbH, Germany
 *  http://www.academic-puma.de
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace AcademicPuma\RestClient\Renderer;

use AcademicPuma\RestClient\Model;
use AcademicPuma\RestClient\Exceptions;
use AcademicPuma\RestClient\Util;
use AcademicPuma\RestClient\Config;
use AcademicPuma\RestClient\Util\BibtexModelUtils;
use AcademicPuma\RestClient\Util\Person;

/**
 * Short description
 *
 * @since 09.07.15
 * @author Sebastian Böttger / boettger@cs.uni-kassel.de
 */
class EndnoteModelRenderer extends ModelRenderer {

    const NEWLINE = PHP_EOL;


    /**
     * Serializes <pre>$posts </pre> recursively into its XML representation
     *
     * @param \AcademicPuma\RestClient\Model\Posts $posts
     *
     * @return string BibTeX formatted posts
     */
    protected function serializePosts(Model\Posts $posts) {
        $bibtexPosts = [];
        foreach ($posts as $post) {
            $bibtexPosts[] = $this->serializePost($post);
            //$this->bibtexPost = '';
        }
        return implode("\n", $bibtexPosts);
    }

    /**
     * Serializes a <pre>$post</pre> recursively into its XML representation
     *
     * @param \AcademicPuma\RestClient\Model\Post $post
     *
     * @return string
     */
    protected function serializePost(Model\Post $post) {

        return $this->serializeResource($post->getResource());
    }

    /**
     * Serializes a <pre>$resource</pre> into its BibTeX representation
     *
     * @param Model\Resource $resource
     *
     * @return string
     * @throws Exceptions\InvalidModelObjectException
     */
    protected function serializeResource(Model\Resource $resource) {
        if ($resource instanceof Model\Bookmark) {
            throw new Exceptions\InvalidModelObjectException("There is no BibTeX representation for resource type Bookmark");
        }
        return $this->serializeEndnote($resource);
    }

    /**
     * @param Model\Bibtex $bibtex
     *
     * @return string
     */
    protected function serializeEndnote(Model\Bibtex $bibtex) {

        $endnoteItem = "";

		$endnoteItem .= "%0 " . (new Util\EndnoteTypeEntrytypeMap())->getEntrytype($bibtex->getEntrytype()) . self::NEWLINE;

		if (self::present($bibtex->getBibtexKey())) {
            $endnoteItem .= "%1 " . BibTexModelUtils::cleanBibTex($bibtex->getBibtexKey()) . self::NEWLINE;
        }

		if (self::present($bibtex->getAuthor())) {

            $persons = Person::createPersonsListFromString($bibtex->getAuthor());
            foreach ($persons as $person) {
                /** @var Person $person */
                $endnoteItem .= "%A " . $person->getLastName() . ", " . $person->getFirstName() . self::NEWLINE;
            }
        }

        if (self::present($bibtex->getBooktitle())) {
            $endnoteItem .= "%B " . BibtexModelUtils::cleanBibtex($bibtex->getBooktitle()) . self::NEWLINE;
        } else if (self::present($bibtex->getSeries())){// TODO: check whether this is correct
            $endnoteItem .= "%B " . BibtexModelUtils::cleanBibtex($bibtex->getSeries()) . self::NEWLINE;
        }

        if (self::present($bibtex->getAddress())) {
            $endnoteItem .= "%C " . BibtexModelUtils::cleanBibtex($bibtex->getAddress()) . self::NEWLINE;
        }

        if (self::present($bibtex->getYear())) {
            $endnoteItem .= "%D " . BibtexModelUtils::cleanBibtex($bibtex->getYear()) . self::NEWLINE;
        }

        if (self::present($bibtex->getEditor())) {

            $persons = Person::createPersonsListFromString($bibtex->getEditor());
            foreach ($persons as $person) {
                $endnoteItem .= "%E " . $person->getLastName() . ", " . $person->getFirstName() . self::NEWLINE;
            }
        }

        if (self::present($bibtex->getPublisher())) {
            $endnoteItem .= "%I " . BibtexModelUtils::cleanBibtex($bibtex->getPublisher()) . self::NEWLINE;
        }

		if (self::present($bibtex->getJournal())) {
            $endnoteItem .= "%J " . BibtexModelUtils::cleanBibtex($bibtex->getJournal()) . self::NEWLINE;
        }

        if (self::present($bibtex->getNumber())) {
            $endnoteItem .= "%N " . BibtexModelUtils::cleanBibtex($bibtex->getNumber()) . self::NEWLINE;
        }

        if (self::present($bibtex->getPages())) {
            $endnoteItem .= "%P " . BibtexModelUtils::cleanBibtex($bibtex->getPages()) . self::NEWLINE;
        }

        if (self::present($bibtex->getMiscField("doi"))) {
            $endnoteItem .= "%R " . BibtexModelUtils::cleanBibtex($bibtex->getMiscField("doi")) . self::NEWLINE;
        }

        if (self::present($bibtex->getTitle())) {
            $endnoteItem .= "%T " . BibtexModelUtils::cleanBibtex($bibtex->getTitle()) . self::NEWLINE;
        }

		if (self::present($bibtex->getUrl())) {
            $endnoteItem .= "%U " . BibtexModelUtils::cleanBibtex($bibtex->getUrl()) . self::NEWLINE;
        }

        if (self::present($bibtex->getVolume())) {
            $endnoteItem .= "%V " . BibtexModelUtils::cleanBibtex($bibtex->getVolume()) . self::NEWLINE;
        }

		if (self::present($bibtex->getBibtexAbstract())) {
            $endnoteItem .= "%X " . BibtexModelUtils::cleanBibtex($bibtex->getBibtexAbstract()) . self::NEWLINE;
        }

        if (self::present($bibtex->getAnnote())) {
            $endnoteItem .= "%Z " . BibtexModelUtils::cleanBibtex($bibtex->getAnnote()) . self::NEWLINE;
        }

        if (self::present($bibtex->getEdition())) {
            $endnoteItem .= "%7 " . BibtexModelUtils::cleanBibtex($bibtex->getEdition()) . self::NEWLINE;
        }

		if (self::present($bibtex->getChapter())) {
            $endnoteItem .= "%& " . BibtexModelUtils::cleanBibtex($bibtex->getChapter()) . self::NEWLINE;
        }

		if (self::present($bibtex->getMiscField("isbn"))) {
            $endnoteItem .= "%@ " . BibtexModelUtils::cleanBibtex($bibtex->getMiscField("isbn")) . self::NEWLINE;
        }

        return $endnoteItem;
    }


    /**
     * Serializes a set of <pre>$tags</pre> into its XML representation
     *
     * @param \AcademicPuma\RestClient\Model\Tags $tags
     */
    protected function serializeTags(Model\Tags $tags) {
        // TODO: Implement serializeTags() method.
    }

    /**
     * Serializes a <pre>$tag</pre> into its XML representation
     *
     * @param \AcademicPuma\RestClient\Model\Tag $tag
     */
    protected function serializeTag(Model\Tag $tag) {
        // TODO: Implement serializeTag() method.
    }

    /**
     * Serializes a set of <pre>$users</pre> into its XML representation
     *
     * @param \AcademicPuma\RestClient\Model\Users $users
     *
     * @throws Exceptions\UnsupportedOperationException
     */
    protected function serializeUsers(Model\Users $users) {
        throw new Exceptions\UnsupportedOperationException(__METHOD__." not supported in this Model Renderer.");
    }

    /**
     * Serializes a <pre>$user</pre> into its XML representation
     *
     * @param \AcademicPuma\RestClient\Model\User $user
     *
     * @throws Exceptions\UnsupportedOperationException
     */
    protected function serializeUser(Model\User $user) {
        throw new Exceptions\UnsupportedOperationException(__METHOD__." not supported in this Model Renderer.");
    }

    /**
     * Serializes a set of <pre>$groups</pre> into its XML representation
     *
     * @param \AcademicPuma\RestClient\Model\Groups $groups
     *
     * @throws Exceptions\UnsupportedOperationException
     */
    protected function serializeGroups(Model\Groups $groups) {
        throw new Exceptions\UnsupportedOperationException(__METHOD__." not supported in this Model Renderer.");
    }

    /**
     * Serializes a <pre>$group</pre> into its XML representation
     *
     * @param \AcademicPuma\RestClient\Model\Group $group
     *
     * @throws Exceptions\UnsupportedOperationException
     */
    protected function serializeGroup(Model\Group $group) {
        throw new Exceptions\UnsupportedOperationException(__METHOD__." not supported in this Model Renderer.");
    }

    /**
     * Serializes a set of <pre>$documents</pre> into its XML representation
     *
     * @param \AcademicPuma\RestClient\Model\Documents $documents
     *
     * @throws Exceptions\UnsupportedOperationException
     */
    protected function serializeDocuments(Model\Documents $documents) {
        throw new Exceptions\UnsupportedOperationException(__METHOD__." not supported in this Model Renderer.");
    }

    /**
     * Serializes a <pre>$document</pre> into its XML representation
     *
     * @param \AcademicPuma\RestClient\Model\Document $document
     *
     * @throws Exceptions\UnsupportedOperationException
     */
    protected function serializeDocument(Model\Document $document) {
        throw new Exceptions\UnsupportedOperationException(__METHOD__." not supported in this Model Renderer.");
    }

    /**
     * @param string $endnoteString
     *
     * @return string
     */
    private function removeLastComma($endnoteString) {

        $cutPos = strpos($endnoteString, ",\n", strlen($endnoteString)-2);
        return substr($endnoteString, 0, $cutPos);
    }

    /**
     * @param mixed $var
     *
     * @return bool
     */
    private static function present($var) {
        return !empty($var);
    }
}
