<?php

/*
 *  restclient-php is a full-featured REST client for PUMA and/or
 *  BibSonomy.
 *
 *  Copyright (C) 2015
 *
 *  Knowledge & Data Engineering Group,
 *  University of Kassel, Germany
 *  http://www.kde.cs.uni-kassel.de/
 *
 *  HothoData GmbH, Germany
 *  http://www.academic-puma.de
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace AcademicPuma\RestClient\Renderer;

use AcademicPuma\RestClient\Model;
use AcademicPuma\RestClient\exceptions\InvalidModelObjectException;

/**
 * Converts model object structure into XML string
 * @author Sebastian Böttger <boettger@cs.uni-kassel.de>
 */
class XMLModelRenderer extends ModelRenderer {

    private static $DOM_POSTS_ATTRIBUTES = ['start', 'end', 'next'];

    private static $DOM_TAGS_ATTRIBUTES = ['start', 'end'];

    private static $DOM_USERS_ATTRIBUTES = ['start', 'end'];

    private static $DOM_GROUPS_ATTRIBUTES = ['start', 'end'];

    private static $DOM_DOCUMENTS_ATTRIBUTES = ['start', 'end'];

    private static $DOM_POST_ATTRIBUTES = ['description', 'postingdate', 'changedate'];

    private static $DOM_USER_ATTRIBUTES = ['name', 'realname', 'email', 'homepage', 'href' ];

    private static $DOM_TAG_ATTRIBUTES = ['name', 'globalcount', 'usercount', 'href'];

    private static $DOM_GROUP_ATTRIBUTES = ['name', 'description', 'href'];

    private static $DOM_RESOURCE_ATTRIBUTES = ['interhash', 'intrahash'];

    private static $DOM_BOOKMARK_ATTRIBUTES = ['title', 'url', 'href'];

    private static $DOM_DOCUMENT_ATTRIBUTES = ['filename','md5hash', 'href'];

    private static $DOM_BIBTEX_ATTRIBUTES = [
            "title",
            "bibtexKey",
            "bKey",
            "misc",
            "bibtexAbstract",
            "entrytype",
            "address",
            "annote",
            "author",
            "booktitle",
            "chapter",
            "crossref",
            "edition",
            "editor",
            "howpublished",
            "institution",
            "organization",
            "journal",
            "note",
            "number",
            "pages",
            "publisher",
            "school",
            "series",
            "volume",
            "day",
            "month",
            "year",
            "type",
            "url",
            "privnote",
            "href"
    ];


    const CLASS_TAGS = '\\AcademicPuma\\RestClient\\Model\\Tags';

    const CLASS_POSTS = '\\AcademicPuma\\RestClient\\Model\\Posts';

    const CLASS_USERS = '\\AcademicPuma\\RestClient\\Model\\Users';

    const CLASS_GROUPS = '\\AcademicPuma\\RestClient\\Model\\Groups';

    const CLASS_DOCUMENTS = '\\AcademicPuma\\RestClient\\Model\\Documents';

    const CLASS_RESOURCE = '\\AcademicPuma\\RestClient\\Model\\Resource';

    const CLASS_BOOKMARK = '\\AcademicPuma\\RestClient\\Model\\Bookmark';

    const CLASS_BIBTEX = '\\AcademicPuma\\RestClient\\Model\\Bibtex';

    const CLASS_TAG = '\\AcademicPuma\\RestClient\\Model\\Tag';

    const CLASS_POST = '\\AcademicPuma\\RestClient\\Model\\Post';

    const CLASS_USER = '\\AcademicPuma\\RestClient\\Model\\User';

    const CLASS_GROUP = '\\AcademicPuma\\RestClient\\Model\\Group';

    const CLASS_DOCUMENT = '\\AcademicPuma\\RestClient\\Model\\Document';

    /**
     *
     * @var \DOMDocument
     */
    private $doc;

    public function __construct() {
        $this->doc = new \DOMDocument();
    }

    /**
     * @param \AcademicPuma\RestClient\Model\Posts $posts
     *
     * @return \DOMElement
     * @throws \AcademicPuma\RestClient\exceptions\InvalidModelObjectException
     */
    protected function serializePosts(Model\Posts $posts) {

        $postsNode = $this->doc->createElement("posts");

        foreach($posts as $post) {
            $postsNode->appendChild(
                $this->serializePost($post)
            );
        }

        $this->appendAttributeNodes(
                $postsNode,
                self::CLASS_POSTS,
                self::$DOM_POSTS_ATTRIBUTES,
                $posts);

        return $postsNode;
    }

    /**
     * @param \AcademicPuma\RestClient\Model\Tags $tags
     *
     * @return \DOMElement
     */
    protected function serializeTags(Model\Tags $tags) {

        $tagsNode = $this->doc->createElement("tags");

        foreach($tags as $tag) {
            $tagsNode->appendChild(
                $this->serializeTag($tag)
            );
        }

        $this->appendAttributeNodes(
                $tagsNode,
                self::CLASS_TAGS,
                self::$DOM_TAGS_ATTRIBUTES,
                $tags);

        return $tagsNode;
    }

    /**
     * @param \AcademicPuma\RestClient\Model\Users $users
     *
     * @return \DOMElement
     */
    protected function serializeUsers(Model\Users $users) {
        $usersNode = $this->doc->createElement("users");

        foreach($users as $user) {
            $usersNode->appendChild(
                $this->serializeUser($user)
            );
        }

        $this->appendAttributeNodes(
                $usersNode,
                self::CLASS_USERS,
                self::$DOM_USERS_ATTRIBUTES,
                $users);

        return $usersNode;
    }

    /**
     * @param \AcademicPuma\RestClient\Model\Groups $groups
     *
     * @return \DOMElement
     */
    protected function serializeGroups(Model\Groups $groups) {
        $groupsNode = $this->doc->createElement("groups");

        foreach($groups as $group) {
            $groupsNode->appendChild(
                $this->serializeGroup($group)
            );
        }

        $this->appendAttributeNodes(
                $groupsNode,
                self::CLASS_GROUPS,
                self::$DOM_GROUPS_ATTRIBUTES,
                $groups);

        return $groupsNode;
    }

    /**
     * @param Model\Documents $documents
     *
     * @return \DOMElement
     */
    protected function serializeDocuments(Model\Documents $documents) {
        $documentsNode = $this->doc->createElement("documents");

        foreach($documents as $document) {
            $documentsNode->appendChild(
                $this->serializeDocument($document)
            );
        }

        $this->appendAttributeNodes(
            $documentsNode,
            self::CLASS_DOCUMENTS,
            self::$DOM_DOCUMENTS_ATTRIBUTES,
            $documents);

        return $documentsNode;
    }

    /**
     * @param \AcademicPuma\RestClient\Model\Post $post
     *
     * @return \DOMElement
     * @throws \AcademicPuma\RestClient\exceptions\InvalidModelObjectException
     */
    protected function serializePost(Model\Post $post) {

        if ($post->getUser() === null || $post->getTag() === null  || $post->getResource() === null) {
            throw new InvalidModelObjectException('Resource, tags or user missing');
        }

        $postNode = $this->doc->createElement("post");

        if ($post->hasGroup()) {
            $postNode->appendChild(
                $this->serializeGroup($post->getGroup())
            );
        }

        $postNode->appendChild(
            $this->serializeUser($post->getUser())
        );

        foreach ($post->getTag() as $tag) {
            $postNode->appendChild(
                $this->serializeTag($tag)
            );
        }

        $postNode->appendChild(
            $this->serializeResource($post->getResource())
        );

        $this->appendAttributeNodes(
                $postNode,
                self::CLASS_POST,
                self::$DOM_POST_ATTRIBUTES,
                $post);

        return $postNode;
    }

    /**
     * @param Model\Document $document
     *
     * @return \DOMElement
     */
    protected function serializeDocument(Model\Document $document) {
        $docNode = $this->doc->createElement("document");
        $this->appendAttributeNodes($docNode, self::CLASS_DOCUMENT, self::$DOM_DOCUMENT_ATTRIBUTES, $document);

        return $docNode;
    }

    /**
     *
     * @param \AcademicPuma\RestClient\Model\User $user
     * @return \DOMNode
     */
    protected function serializeUser(Model\User $user) {

        $userNode = $this->doc->createElement("user");

        $newNode = $this->doc->appendChild($userNode);

        if ($user->hasGroups()) {

            $newNode->appendChild(
                $this->serializeGroups($user->getGroups())
            );
        }
        $this->appendAttributeNodes(
                $newNode,
                self::CLASS_USER,
                self::$DOM_USER_ATTRIBUTES,
                $user);

        return $newNode;
    }

    /**
     * @param \AcademicPuma\RestClient\Model\Tag $tag
     *
     * @return \DOMNode
     */
    protected function serializeTag(Model\Tag $tag) {

        $tagNode = $this->doc->createElement("tag");

        $newNode = $this->doc->appendChild($tagNode);

        $this->appendAttributeNodes(
                $newNode,
                self::CLASS_TAG,
                self::$DOM_TAG_ATTRIBUTES,
                $tag);

        if ($tag->hasSubTags()) {
            $subTags = $this->doc->createElement("subTags");

            foreach ($tag->getSubTags() as $subTag) {
                //append each subTag to subTags-node
                $subTags->appendChild($this->serializeTag($subTag)); //recursive
            }
            $tagNode->appendChild($subTags); //append subTags-node to the tag node
        }

        return $newNode;
    }

    /**
     * @param \AcademicPuma\RestClient\Model\Group $group
     *
     * @return \DOMNode
     */
    protected function serializeGroup(Model\Group $group) {

        $groupNode = $this->doc->createElement("group");

        $newNode = $this->doc->appendChild($groupNode);

        $this->appendAttributeNodes(
                $newNode,
                self::CLASS_GROUP,
                self::$DOM_GROUP_ATTRIBUTES,
                $group);

        return $newNode;
    }

    protected function serializeResource(Model\Resource $resource) {

        $resourceNode = null;

        if ($resource instanceof Model\Bookmark) {
            $resNode = $this->doc->createElement('bookmark');
            $resourceNode = $this->doc->appendChild($resNode);
            $this->appendAttributeNodes($resourceNode, self::CLASS_RESOURCE, self::$DOM_RESOURCE_ATTRIBUTES, $resource);
            $this->appendAttributeNodes($resourceNode, self::CLASS_BOOKMARK, self::$DOM_BOOKMARK_ATTRIBUTES, $resource);
        }
        if ($resource instanceof Model\Bibtex) {
            $resNode = $this->doc->createElement('bibtex');
            $resourceNode = $this->doc->appendChild($resNode);
            $this->appendAttributeNodes($resourceNode, self::CLASS_RESOURCE, self::$DOM_RESOURCE_ATTRIBUTES, $resource);
            $this->appendAttributeNodes($resourceNode, self::CLASS_BIBTEX, self::$DOM_BIBTEX_ATTRIBUTES, $resource);
        }
        return $resourceNode;
    }


    private function appendAttributeNodes(\DOMElement &$domNode, $modelClassName, array $attributeList, $modelObject) {

        $reflClass = new \ReflectionClass($modelClassName);

        foreach($attributeList as $attr) {

            try {
                $method = $reflClass->getMethod('get' . ucfirst($attr));
                $retVal = $method->invoke($modelObject);
            } catch(\ReflectionException $e) {
                continue;
            }
            if ($retVal !== '' && $retVal !== null) {
                $domNode->setAttribute($attr, $retVal);
            }

        }
    }

    /**
     *
     * @param \AcademicPuma\RestClient\Model\ModelObject $model
     * @return string XML formatted string
     */
    public function render(Model\ModelObject $model) {
        return $this->renderXML(parent::render($model));
    }

    /**
     * @param \DOMNode $node
     *
     * @return string
     */
    private function renderXML(\DOMNode $node = null) {

        if (null !== $node) {
            $rootNode = $this->doc->createElement("bibsonomy");
            $rootNode->appendChild($node);
            return $this->doc->saveXML($rootNode, LIBXML_NOBLANKS);
        }
        return $this->doc->saveXML();
    }

}
