<?php

/*
 *  restclient-php is a full-featured REST client for PUMA and/or
 *  BibSonomy.
 *
 *  Copyright (C) 2015
 *
 *  Knowledge & Data Engineering Group,
 *  University of Kassel, Germany
 *  http://www.kde.cs.uni-kassel.de/
 *
 *  HothoData GmbH, Germany
 *  http://www.academic-puma.de
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace AcademicPuma\RestClient\Util\Collection;

use AcademicPuma\RestClient\Config\Sorting;
use AcademicPuma\RestClient\Model;
use AcademicPuma\RestClient\Exceptions;

/**
 * Comparator for sort function of <code>Model\Posts</code>
 *
 * @author Sebastian Böttger / boettger@cs.uni-kassel.de
 */
class PostComparator {

    public static $COMPARABLE_ATTRIBUTES = ['author', 'title', 'year', 'entrytype', 'dblp'];

    protected $sortingKey;

    protected $sortingOrder;

    protected $userDefinedTypeOrder;

    public function __construct($sortingKey, $sortingOrder = Sorting::ORDER_ASC, $userDefinedTypeOrder = []) {
        $this->sortingKey = $sortingKey;
        $this->sortingOrder = $sortingOrder;
        $this->userDefinedTypeOrder = $userDefinedTypeOrder;
    }

    /**
     * @param Model\Post $a
     * @param Model\Post $b
     *
     * @return mixed
     * @throws Exceptions\InvalidSortingTypeException
     */
    public function compare(Model\Post $a, Model\Post $b) {

        if ($a->getResource() instanceof Model\Bookmark && $this->sortingKey !== 'title') {

            throw new Exceptions\InvalidSortingTypeException("Attribute '.$this->sortingKey.' is not a valid
                sorting key for Bookmarks. Use 'title' instead.");
        } else if (!in_array($this->sortingKey, self::$COMPARABLE_ATTRIBUTES)) {

            throw new Exceptions\InvalidSortingTypeException("Attribute '.$this->sortingKey.' is not a valid
                sorting key for Bibtex. Use ".implode(', ', self::$COMPARABLE_ATTRIBUTES)." instead.");
        }

        if ($this->sortingKey === 'entrytype' && !empty($this->userDefinedTypeOrder)) {

            /** @var Model\Bibtex $bibtex_a */
            $bibtex_a = $a->getResource();
            /** @var Model\Bibtex $bibtex_b */
            $bibtex_b = $b->getResource();

            return $bibtex_a->compareEntrytype($bibtex_b, $this->userDefinedTypeOrder);
        }

        if ($this->sortingKey === 'dblp') {

            /** @var Model\Bibtex $bibtex_a */
            $bibtex_a = $a->getResource();
            /** @var Model\Bibtex $bibtex_b */
            $bibtex_b = $b->getResource();

            return $bibtex_a->compareDBLPFormat($bibtex_b);
        }

        return ($this->sortingOrder === Sorting::ORDER_ASC) ?
                $a->getResource()->compare($b->getResource(), $this->sortingKey) :
                $b->getResource()->compare($a->getResource(), $this->sortingKey);

    }
}